#ifndef _CLSHARED_H

#define _CLSHARED_H

#define CL_LOGIC_API_VERSION	1341

class CSafeModelPtr;
typedef struct rda_s rda_t;
typedef struct texShader_s texShader_t;
typedef struct rsprite_s rsprite_t;
typedef struct cubicGridSpace_s cubicGridSpace_t;
typedef struct waveFile_s waveFile_t;
typedef struct particleInstance_s particleInstance_t;
typedef struct particleSystem_s particleSystem_t;

typedef struct clientString_s
{
	char		p[64];
} clientString_t;

typedef struct texContainer_s
{
	bool			inuse;
	bool			pad[3];
	texShader_t		*shader;
} texContainer_t;

typedef enum
{
	RGL_PRIM_QUADS,
	RGL_PRIM_TRIANGLES,
	RGL_PRIM_LINES,
	NUM_RGL_PRIMS
} rglPrimTypes;

typedef enum
{
	FLARETEX_CORONA,
	FLARETEX_MID,
	FLARETEX_MID2,
	FLARETEX_MID3,
	FLARETEX_BURST,
	FLARETEX_END,
	FLARETEX_NUM
} flareTextures_e;

typedef struct lensFlare_s
{
	engineTime_t		inViewTime;

	int					numMidSections;
	float				length;
	float				sizeScale;
	texContainer_t		textures[FLARETEX_NUM];
} lensFlare_t;

typedef struct riiEntry_s
{
	char			*name;
	float			tc[4];
} riiEntry_t;

typedef struct rimageIndex_s
{
	int				numImages;
	int				pageWidth;
	int				pageHeight;

	strHash_t		*imgHash;
	cntArray_t		*imgArray;
} rimageIndex_t;

typedef struct rm2ListItem_s
{
	texContainer_t	icon;
	riiEntry_t		*iconRiiEntry;
	char			*text;
	char			*colText;
	bool			selectable;
} rm2ListItem_t;

#define TROPHYTYPE_BRONZE		0
#define TROPHYTYPE_SILVER		1
#define TROPHYTYPE_GOLD			2
typedef struct clientTrophy_s
{
	char				*trophyID;
	int					trophyType;
	bool				isSecret;

	char				*trophyName;
	char				*trophyDesc;

	bool				isUnlocked;
} clientTrophy_t;

typedef struct ambientLight_s
{
	//ambientLightInteraction_t	*interactions;
	//int						numInteractions;
	float						worldTranslation[3];
	//float						mins[3];
	//float						maxs[3];

	float						scale[3];
	int							glTex;
} ambientLight_t;

typedef struct modelLightInteraction_s modelLightInteraction_t;
typedef struct sceneLight_s
{
	float			pos[3];
	float			color[3];
	float			radius;
	bool			shadows;
	ambientLight_t	*ambientLight;
	int				staticIndex;
	bool			hasFrameScissor;
	bool			doFrameScissor;
	float			frameScissor[6];
	int				lightFlags;
	viewFrustum_t	*spotFrust;
	texContainer_t	spotTex;
	bool			noVolumes;
	CSafeModelPtr	*noShadowModels[8];
	int				numNoShadowModels;
	float			lightMatrix[16];
	bool			hasShadowMaps;

	volatile int			frameCalculated;
	int						threadIndex;
	modelLightInteraction_t	*inter;
	int						numInter;
	modelLightInteraction_t	**modShadowInter;
	int						numModInter;
} sceneLight_t;

typedef struct trailPoint_s
{
	float			pos[3];
	engineTime_t	time;
} trailPoint_t;

#define ASYNC_TRAIL_GEO

typedef struct trailVert_s
{
	float			pos[3];
#ifdef _IPHONE
	BYTE			color[4];
#else
	float			color[4];
#endif
	float			tc[2];
	float			normal[3]; //not actually used for drawing on iphone
} trailVert_t;

#ifdef ASYNC_TRAIL_GEO
typedef struct asyncTrailData_s
{
	int				finalUsedIdxs;
	trailVert_t		*finalVerts;
	WORD			*finalIndices;
	int				drawType;
	bool			isCulled;
	volatile bool	isCalculated;
} asyncTrailData_t;
#endif

typedef struct renderTrail_s
{
	trailPoint_t	*points;
	int				numPoints;
	float			width;
	engineTime_t	segmentLife;
	engineTime_t	segmentFade;
	float			segmentRise;

	float			*fadeMap;
	int				fadeNum;
	float			*tcMap;
	int				tcNum;

	float			*worldNormals;
	bool			batchAllocated;

	float			blendRGBA[4];
	bool			hasRGBA;

	float			mins[3];
	float			maxs[3];
	int				flatTrail;
} renderTrail_t;

typedef struct spriteBatchInfo_s
{
	float			pos[3];
	float			ang[3];
	float			rgba[4];
	float			scale[2];
	float			*tc;
} spriteBatchInfo_t;

typedef struct simpleSpriteTrn_s simpleSpriteTrn_t;

#define SSPRITEFLAG_NODEPTHWRITE		(1<<0)
#define SSPRITEFLAG_NODEPTHTEST			(1<<1)
#define SSPRITEFLAG_TWOSIDED			(1<<2)
#define SSPRITEFLAG_DEPTHBIAS			(1<<3)
#define SSPRITEFLAG_HEALTHBAR			(1<<4) //ultimate hax, means top 8 bits of ssFlags contain health fraction
#define SSPRITEFLAG_PARTICLEBASED		(1<<5)

typedef struct simpleSprite_s
{
	//float			verts[3*4];
	//float			tc[2*4];
	float			*verts;
	float			*tc;
	float			scaleX;
	float			scaleY;
//	bool			noDepthWrite;
//	bool			noDepthTest;
//	bool			twoSided;
	int				ssFlags;
	spriteBatchInfo_t *batchInfo;
} simpleSprite_t;

typedef struct genericDraw_s
{
	int				primType;

	int				numVerts;
	float			*verts;
	float			*texCoords;
	float			*colors;

	float			pos[3];
	float			ang[3];

	bool			freeBuffers;
	bool			forceNoDepth;

	genericDraw_s	*next;
} genericDraw_t;

typedef struct textureRemap_s
{
	texContainer_t		src;
	texContainer_t		dst;
} textureRemap_t;

#ifdef _IPHONE
#define _TRIM_CLIENT_OBJ
#endif

typedef struct drawObject_s
{
	float				pos[3];
	float				ang[3];

	CSafeModelPtr		*model;
	rda_t				*anim;
	float				modelScale[3];

#ifndef _IPHONE
	cubicGridSpace_t	*cgrid;
	int					cgridNum;

	rsprite_t			*rspr;
	int					rsprAng;
#endif
	
	int					blendFrame; //applicable only for skeletal animation
	int					frame;
	float				lerpAmount; //applicable only for skeletal animation

	renderTrail_t		*trail;
	int					numTrails;
#ifdef ASYNC_TRAIL_GEO
	asyncTrailData_t	*trailAsync;
#endif
	simpleSprite_t		*ssprite;
	int					sspriteBatchNum;
	simpleSpriteTrn_t	*sspriteTransform;

#ifndef _IPHONE	
	lensFlare_t			*lensFlare;
#endif
	
	texContainer_t		texture;
	int					deforms;
	float				deformPropA;
	float				deformPropB;
	float				deformPropC;
	float				deformPropD;
	
#ifndef _IPHONE	
	float				postTrailLen;
	float				postTrailMul;
	float				postTrailDir[3];
	int					postStencil;
#endif

	int					blendSrc;
	int					blendDst;
	float				blendRGBA[4];

	int					staticIndex;


	int					gridColorFrame;
	float				gridColor[3];
	float				gridColorSec[3];
	float				gridNormal[3];
	float				gridNormalWorld[3];
	float				groundZ;

	float				modelMat[16];
	float				modelToLocalMat[16];
	modelMatrix_t		invModelMat;
	/*
	float				testMins[3];
	float				testMaxs[3];
	*/

	textureRemap_t		*texRemaps;
	int					numTexRemaps;

	int					forceSortVal;

#ifndef _IPHONE		
	bool				rsprNoOri;
	bool				pad[3];
#endif
	
	bool				angAbsolute;
	bool				objDynamicIllumination;	
	bool				noShadows;
	bool				skipDepth;
	
	bool				groundZValid;
	bool				frameCulled;
	bool				didFrameCulling;
	bool				hasRepInfo;
	
	/*
	//SSHX THIS MUST REMAIN IN THE LAST 160 BYTES
	int					hasModelMat;
	float				modelMat[16];
	int					hasModelToLocalMat;
	float				modelToLocalMat[16];
	float				testMins[3];
	float				testMaxs[3];
	*/
} drawObject_t;

typedef struct clientObject_s
{
	bool			active;

	CSafeModelPtr	*model;
	rda_t			*anim;
	int				animStrIndex;

#ifndef _TRIM_CLIENT_OBJ
	lensFlare_t		flare;
#endif
	renderTrail_t	*trail;
	renderTrail_t	*boneTrail;
	int				boneTrailBoneIndex;
	renderTrail_t	*bladeTrail;
	int				bladeTrailBoneIndex;
	int				bladeTrailBone2Index;
#ifndef _TRIM_CLIENT_OBJ
	float			postPos[3];
	float			postFade;
#endif
	engineTime_t	deathFxTime;
	engineTime_t	spawnFxTime;
	engineTime_t	otherFxTime;
	engineTime_t	deathPclTime;
	engineTime_t	anotherPclTime;
	bool			hasBoneOverride;
	bool			hasPumpkinHead;
	bool			hasBonePush;
	bool			hasIKBall;
	bool			hasIKHair;

	float			projPos[3];
	float			projAng[3];
	float			projScale[3];

	bool			postAdd; //add after everything else

	BYTE			*objectData;
	bool			checkedProps;

	engineTime_t	clAnimTime;

	engineTime_t	clLerpAnimTime;
	int				clLerpAnimFrame;
	int				clLerpAnimBlendFrame;

	bool			hasSoundData;
	float			soundData;

	textureRemap_t	texRemaps[4];
	int				numTexRemaps;
	bool			hasObjTexmap;

	drawObject_t	obj;
	gameObjNet_t	net;
} clientObject_t;

typedef struct menuScript_s menuScript_t;
typedef struct mscriptNode_s mscriptNode_t;

//max args in a single script instruction
#define MAX_MENUSCRIPT_ARGS		8

typedef struct menuModel_s
{
	CSafeModelPtr		*model;
	rda_t				*modelAnim;
	int					modelAnimStart;
	int					modelAnimEnd;
	int					modelAnimFrame;
	int					modelAnimBlendFrame;
	engineTime_t		modelAnimLerpTime;
	char				*modelName;
	char				*modelAnimName;
	float				modelOfs[3];
	float				modelAng[3];
} menuModel_t;

typedef struct menu_s
{
    char				name[64];
	char				*text;
	int					x;
	int					y;
	int					w;
	int					h;
	texContainer_t		texture;
	int					drawTypeID;

	texContainer_t		texBtnUp;
	texContainer_t		texBtnDown;
	int					buttonState;
	int					backButton;

	int					textOffX;
	int					textOffY;
	int					inputTextOffX;
	int					inputTextOffY;

	int					blendSrc;
	int					blendDst;
	float				blendRGBA[4];

	bool				selectable;
	bool				ratioCorrect;
	bool				noLineWrap;
	int					textSelector;
	char				*textSelPtr;
	int					textSelLineCount;
	float				fontScaleX;
	float				fontScaleY;

	bool				takeInput;
	char				inputStr[64];

	bool				inuse;
	bool				server; //if true, then input etc should be sent to the server.

	char				activateScript[64];
	char				onEscape[64];
	char				onSort[64];
	bool				draw;

	bool				isSorting;
	int					sortLine;

	menuModel_t			mdl;

	int					touchIndex;

	texContainer_t		ptrTexture;
} menu_t;

#define MAX_RM2_TEXTSOURCE_ARGS		4
#define MAX_RM2_TEXTSOURCE_LEN		512
#define MAX_RM2_TEXTRET_LEN			4096

typedef struct rm2SrcFunc_s
{
	char			funcName[MAX_RM2_TEXTSOURCE_LEN];
	char			args[MAX_RM2_TEXTSOURCE_ARGS][MAX_RM2_TEXTSOURCE_LEN];
	int				numArgs;
} rm2SrcFunc_t;

typedef struct menuPointer_s
{
	menu_t			*selected;
} menuPointer_t;

typedef struct mscriptArg_s
{
	char			p[64];
} mscriptArg_t;

typedef struct mscriptHandler_s
{
	char			*name;
	bool			(*handlerFunc)(mscriptNode_t *node, menuScript_t *mscript);
	int				argNum;
} mscriptHandler_t;

typedef struct mscriptNode_s
{
	mscriptArg_t		menuArgs[MAX_MENUSCRIPT_ARGS];
	mscriptHandler_t	*nodeHandler;

	mscriptNode_t		*next;
} mscriptNode_t;

typedef struct menuScript_s
{
	char			name[64];

	engineTime_t	scriptTime;
	mscriptNode_t	*nodes;
	mscriptNode_t	*curNode;

	bool			inuse;
} menuScript_t;

#define CLLOCALVAR_TRAVELDEST		0
#define CLLOCALVAR_OBJECTIVEDEST	1

typedef struct sharedCLFunctions_s
{
	//called back into by the engine
	void					(*LCl_Frame)(engineTime_t time, int numObj, int localCl, bool clientInterpolate,
										bool isMulti, float musicFrac, float *viewPos, float *viewAng);
	void					(*LCl_Draw2D)(viewFrustum_t *viewFrust);
	void					(*LCl_DrawPostMenu)(viewFrustum_t *viewFrust);
	void					(*LCl_UnlockTrophy)(char *trophyName);
	void					(*LCl_GetListSource)(char *sourceName, int *numListItems, rm2ListItem_t **listItemsPtr, int *setIndex);
	char					*(*LCl_RM2FunctionEval)(rm2SrcFunc_t *rfunc);
	bool					(*LCl_RM2CheckUsable)(char *evName);
	void					(*LCl_GetTrophyList)(int *numTrophies, clientTrophy_t **trophyPtr);
	bool					(*LCl_ShouldDrawSkybox)(void);
	void					(*LCl_DrawSkybox)(void);
	void					(*LCl_SetLocalClVar)(int var, int val);
	int						(*LCl_GetLocalClVar)(int var);
	bool					(*LCl_CanPause)(void);
	void					(*LCl_CacheResources)(void);
	void					(*LCl_NetEvent)(clientEventTypes_e e, BYTE *data, int dataSize);
	void					(*LCl_FreshClientObject)(clientObject_t *obj);
	void					(*LCl_HandleCustomObjData)(clientObject_t *obj, BYTE *customData, WORD customDataSize);
	void					(*LCl_UserMenu)(const char *callName, mscriptNode_t *node, menuScript_t *mscript);
	bool					(*LCl_UserSort)(const char *sortName, menu_t *menu);

	//math routines
	void					(*Math_CalcPlaneEq)(float *x, float *y, float *z, float *planeEq);
	float					(*Math_VecNorm2)(float *v);
	float					(*Math_VecNorm)(float *v);
	float					(*Math_PlaneNorm)(float *plane);
	float					(*Math_VecLen)(float *a);
	float					(*Math_PlaneDist)(const float *pl, const float *p);
	void					(*Math_ProjectOntoPlane)(const float *plane, const float *pos, float *out);
	void					(*Math_VecCopy)(float *a, float *out);
	void					(*Math_VecSub)(float *a, float *b, float *out);
	void					(*Math_VecAdd)(float *a, float *b, float *out);
	void					(*Math_VecScale)(float *a, float scale);
	float					(*Math_Max2)(float a, float b);
	float					(*Math_Max3)(float a, float b, float c);
	void					(*Math_TransformPointByMatrix)(modelMatrix_t *matrix, float *in, float *out);
	void					(*Math_TransformPointByMatrix3x3)(modelMatrix_t *matrix, float *in, float *out);
	void					(*Math_TransformPoint)(float *basePos, float *scale, float *eAngles, float *in, float *out);
	void					(*Math_VecMA)( float *veca, float scale, float *vecb, float *vecc);
	void					(*Math_VecToAngles)(const float *value1, float *angles);
	void					(*Math_TransformPoint2)(float *basePos, float *scale, float *eAngles, float *in, float *out);
	void					(*Math_MatrixInverse)(modelMatrix_t *in, modelMatrix_t *out);
	void					(*Math_MatrixMultiply)(modelMatrix_t *in, modelMatrix_t *in2, modelMatrix_t *out);
	void					(*Math_MatrixMultiply3x3)(modelMatrix_t *in, modelMatrix_t *in2, modelMatrix_t *out);
	void					(*Math_MatrixMultiply4x4)(fourxMatrix_t *in, fourxMatrix_t *in2, fourxMatrix_t *out);
	void					(*Math_FromToRotationMatrix)(float *from, float *to, modelMatrix_t *mat);
	void					(*Math_PointMatrix)(modelMatrix_t *mat, float *dir, int axis, bool neg);
	void					(*Math_RotateMatrix)(modelMatrix_t *mat, float ang, float x, float y, float z);
	void					(*Math_TranslateMatrix)(modelMatrix_t *mat, float *v);
	void					(*Math_AnglesToMat)(const float *incAngles, modelMatrix_t *mat);
	void					(*Math_CrossProduct)(const float *v1, const float *v2, float *cross);
	float					(*Math_DotProduct)(const float *v1, const float *v2);
	float					(*Math_AngleMod)(float angle);
	void					(*Math_InterpolateVector)(float *old, float *cur, float val, float tresh, float *out);
	float					(*Math_BlendAngle)(float in, float goal, float factor);
	float					(*Math_BlendAngleLinear)(float in, float goal, float amount);
	void					(*Math_InterpolateAnglesLinear)(float *angles, float *goalAngles, float amount);
	void					(*Math_InterpolateAnglesNonLinear)(float *angles, float *goalAngles, float amount);
	void					(*Math_AngleVectors)(float *angles, float *forward, float *right, float *up);
	void					(*Math_AxisForNormal)(float *normal, float *fwd, float *right, float *up);
	void					(*Math_LerpMatrices)(modelMatrix_t &preMat, modelMatrix_t &postMat, float lerpFrac, modelMatrix_t &outMat, bool nonUniform);
	void					(*Math_CreateMatrixFromOrientation)(float *origin, float *angles, modelMatrix_t *mat);
	void					(*Math_RotationMatrix)(float phi, int axis, modelMatrix_t *mat);
	bool					(*Math_BoxesOverlap)(float *mins1, float *maxs1, float *mins2, float *maxs2);
	void					(*Math_ExpandBounds)(float *baseMins, float *baseMaxs, float *mins, float *maxs);
	void					(*Math_CreateFrustum)(float fov, float *pos, float *ang, viewFrustum_t *out);
	void					(*Math_CreateFrustumForViewAxis)(float *pos, float *axis, viewFrustum_t *out, float fovX, float fovY, float zNear, float zFar);
	void					(*Math_GetFrustumPoints)(viewFrustum_t *frustum, float points[8][3], modelMatrix_t *frustMat);
	bool					(*Math_PointBoxOverlap)(float *point, float *boxMins, float *boxMaxs);
	void					(*Math_BoundsFromPoints)(float *outMins, float *outMaxs, float *points, int numPoints);
	int						(*Math_PointRelativeToPlane)(float *pos, float *side, float *planeEq);
	void					(*Math_GetBoundsPoints)(float *mins, float *maxs, float *bp, float *baxis);
	bool					(*Math_PointInFrustum)(viewFrustum_t *frustum, float *point, float radius);
	int						(*Math_BoxInFrustum)(viewFrustum_t *frustum, float *mins, float *maxs);
	void					(*Math_MatToAngles)(float *angles, const modelMatrix_t *mat);
	void					(*Math_MatToAngles2)(float *angles, const modelMatrix_t *mat);
	float					(*Math_ConstantLerp)(float y0, float y1, float amount);
	float					(*Math_LinearLerp)(float y0, float y1, float frac);
	float					(*Math_BilinearLerp)(float y0, float y1, float y2, float y3, float fracX, float fracY);
	float					(*Math_TriLerp)(float y0, float y1, float y2, float fracX, float fracY, float fracZ);
	float					(*Math_CubicLerp)(float y0, float y1, float y2, float y3, float frac);
	float					(*Math_GetMappedValue)(float perc, float *values, int numValues);

	//file system
	BYTE					*(*FileSys_ReadWholeFile)(const char *filename, int *fileLen, int sizePad);	
	int						(*FileSys_OpenFile)(const char *filename, int mode1, int mode2);
	void					(*FileSys_CloseFile)(int handle);
	int						(*FileSys_GetLen)(int handle);
	bool					(*FileSys_ReadFile)(int handle, void *out, int len);
	bool					(*FileSys_WriteFile)(int handle, void *in, int len);
	bool					(*FileSys_Read)(int handle, void *out, int len);
	bool					(*FileSys_Write)(int handle, void *in, int len);
	char					*(*FileSys_GetFileList)(const char *searchPath, const char *searchType);
	int						(*FileSys_WriteDirect)(const char *filename, void *in, int len);

	void					*(*Common_RCMalloc)(unsigned int size); //malloc
	void					(*Common_RCFree)(void *ptr); //free
	int						(*Common_ProcessTouchEvents)(void (*touchEvProc)(int touchEv, appTouch_t *touch));
	void					(*Common_SetAnalogs)(WORD *analogs);
	void					(*Common_SetButton)(int buttonIdx, BYTE val);
	void					(*Common_GetAnalogs)(WORD *analogs);
	BYTE					(*Common_GetButton)(int buttonIdx);
	bool					(*Common_CanPauseToMenu)(void);
	BYTE					*(*Common_GetEngineGlobalVar)(char *varName);

	//resource functions
	BYTE					*(*RCO_GetEntryForObject)(const char *objName);
	const char				*(*RCO_GetValForKey)(BYTE *aobj, const char *keyName);
	void					(*RCO_SetValForKey)(BYTE *aobj, const char *keyName, const char *value);
	int						(*Img_TextureLoad)(const char *name, texContainer_t *tx);
	//void					(*Img_FreeRImageIndex)(rimageIndex_t *rii);
	riiEntry_t				*(*Img_GetRIIEntry)(char *entryName, rimageIndex_t *rii);
	rimageIndex_t			*(*Img_ParseRImageIndex)(const char *fileName);
	CSafeModelPtr			*(*Model_Allocate)(const char *filename);
	void					(*Model_Free)(CSafeModelPtr *model);
	char					*(*Model_GetName)(CSafeModelPtr *model);
	void					(*Model_ProjectDecal)(drawObject_t *modelSceneObj, float *start, float *end, float decalSize, texContainer_t *decalTex, int lifeTime, int fadeTime);
	void					(*Model_SetBoneMats)(CSafeModelPtr *model, modelMatrix_t *boneMats, int numBones);
	rda_t					*(*Anim_Allocate)(const char *filename, bool decompress);
	waveFile_t				*(*S_CacheSound)(const char *filename);
	void					(*S_PlayWave)(waveFile_t *wave, float *pos, float volume, bool neverStomp, int attachClientObj, int forceChannel);
	particleInstance_t		*(*ClPcl_CreateParticles)(const char *pclName, float *pos, float *angles, int clOwner, int clBone);
	particleInstance_t		*(*ClPcl_CreateParticlesOnBone)(const char *pclName, float *pos, float *angles, int clOwner, const char *clBone);
	void					(*ClPcl_CacheParticleSystemAssets)(const char *pclName, particleSystem_t *pcl);

	//gl interface
	void					(*GL_AddDrawObject)(drawObject_t *obj);
	void					(*GL_AddSkyObject)(drawObject_t *obj);
	void					(*GL_AddSceneLight)(sceneLight_t *light);
	void					(*GL_AddGenericObject)(genericDraw_t *obj);
	void					(*GL_BindTexContainer)(texContainer_t *tx);
	float					(*GL_RatioCorrectWidth)(float f);
	float					(*GL_ScaleBasedOnResX)(float f);
	float					(*GL_ScaleBasedOnResY)(float f);
	float					(*GL_ScaleBasedOnResW)(float f);
	float					(*GL_ScaleBasedOnResH)(float f);
	void					(*GL_VertProgramBegin)(bool untextured);
	void					(*GL_VertProgramEnd)(void);
	void					(*GL_PrepareForDraw)(void);
	void					(*GL_DrawRect)(float x, float y, float w, float h, texContainer_t *tex, bool flipH, bool flipV);
	void					(*GL_DrawRectRot)(float x, float y, float w, float h, texContainer_t *tex, float angle);
	void					(*GL_DrawSubRect)(float x, float y, float w, float h, texContainer_t *tex, bool flipH, bool flipV, float *tc);
	void					(*GL_DrawSubRectRot)(float x, float y, float w, float h, texContainer_t *tex, float angle, float *tc);
	float					(*GL_DrawRawText)(float x, float y, float scaleX, float scaleY,
								char *txPtr, bool isInput, float lineWrapLen, float *fontColor);
	float					(*GL_DrawRawTextEx)(float x, float y, float scaleX, float scaleY,
								char *txPtr, bool isInput, float lineWrapLen, float *fontColor,
								float useZ, bool forceControlCodes, bool reallyDraw, float *outW, float *outH,
								bool textSizeCall);
	void					(*GL_GetTextSize)(char *txPtr, float scaleX, float scaleY, float &sizeW, float &sizeH, float lineWrapLen, bool forceControlCodes);
	void					(*GL_SetFlash)(engineTime_t duration, engineTime_t time, float *clr);
	void					(*GL_SetViewTrails)(float amount, float decay);
	void					(*GL_SetBloomMod)(float amount);
	void					(*GL_SetFadein)(engineTime_t dur);
	void					(*GL_WorldToScreen)(float *worldPos, float *screenPos);
	void					(*RGL_Begin)(int primType);
	void					(*RGL_End)(void);
	void					(*RGL_BatchBegin)(bool quads);
	void					(*RGL_BatchEnd)(void);
	void					(*RGL_Vertex3fv)(float *v);
	void					(*RGL_Vertex3f)(float a, float b, float c);
	void					(*RGL_Vertex2fv)(float *v);
	void					(*RGL_Vertex2f)(float a, float b);
	void					(*RGL_TexCoord2fv)(float *tc);
	void					(*RGL_TexCoord2f)(float s, float t);
	void					(*RGL_Color4fv)(float *clr);
	void					(*RGL_Color4f)(float r, float g, float b, float a);
	void					(*RGL_Color3fv)(float *clr);
	void					(*RGL_Color3f)(float r, float g, float b);
	void					(*RGL_Enable)(int cap);
	void					(*RGL_Disable)(int cap);
	void					(*RGL_BlendFunc)(int srcBlend, int dstBlend);
	void					(*GL_SetFragmentFog)(float *color, float startDist, float endDist, int fogBlendMode);
	void					(*ClGfx_Rotate)(float ang, float x, float y, float z);
	void					(*ClGfx_Translate)(float x, float y, float z);
	void					(*ClGfx_Scale)(float x, float  y, float z);
	void					(*ClGfx_OrthoMatrix)(float left, float right, float bottom, float top, float zNear, float zFar);
	void					(*ClGfx_LoadIdentity)(void);
	void					(*ClGfx_LoadMatrix)(float *m);
	void					(*ClGfx_MultMatrix)(float *m);
	void					(*ClGfx_PushMatrix)(void);
	void					(*ClGfx_PopMatrix)(void);
	void					(*ClGfx_MatrixMode)(int mode);
	void					(*ClGfx_GetFloat)(int e, float *outBuf);

	//client
	void					(*Cl_AddAllObjects)(void);
	void					*(*Cl_FrameAlloc)(int size);
	void					(*ClPcl_SetParticleTimeScale)(float timescale);
	void					(*Cl_LocalSaveGame)(char *saveName);
	void					(*Cl_LocalLoadGame)(char *saveName);
	void					(*Cl_GetEnOp)(int opNum, void *opOut);

	//menu interface
	menu_t					*(*Menu_GetMenu)(char *menuName);
	menu_t					*(*Menu_GetActiveMenu)(char *menuName);
	menuPointer_t			*(*Menu_GetMenuPointer)(void);
	void					(*Menu_ParseFile)(char *menuFile);
	bool					(*Menu_StartScript)(char *scriptName);
	bool					(*Menu_StopScript)(char *scriptName);
	void					(*Menu_SetDraw)(menu_t *menu, bool draw);
	bool					(*Menu_AnyActiveMenus)(void);
	bool					(*Menu_WantTouchInput)(void);

	bool					(*RMenu_IsStackClear)(void);
	int						(*RMenu_GetScrollSel)(char *scrollName);

	//network interface
	void					(*Net_DoChat)(const char *str);
	int						(*Net_CreateDelta)(void *previous, void *current, int dataSize, BYTE *deltaOut, int deltaOutSize);
	void					(*Net_ParseDelta)(void *current, int dataSize, BYTE *deltaIn);
	void					(*Net_SetClientUserData)(BYTE *data, int dataSize);
	void					(*Net_SendSvEventType)(serverEventTypes_e eventType, void *data, int dataSize);

	//input
	bool					(*DInput_ButtonPressed)(int button);
	void					(*DInput_GetAnalogs)(WORD *analogs);

	//threading
	void					(*JT_DoJob)(void (*job)(void *userData), void *userData, jobHandle_t *jobHandle);
	bool					(*JT_JobDone)(jobHandle_t *jobHandle, bool stall);
	void					(*Thread_EnterCritical)(int critIndex);
	void					(*Thread_LeaveCritical)(int critIndex);

	//parsing
	textParser_t			*(*Parse_InitParser)(char *rawText);
	textParser_t			*(*Parse_InitParserFromFile)(const char *filename);
	void					(*Parse_FreeParser)(textParser_t *parser);
	void					(*Parse_EnableInclude)(textParser_t *parser, const char *includeKey);
	void					(*Parse_EnableDefines)(textParser_t *parser, const char *defineKey);
	bool					(*Parse_GetNextToken)(textParser_t *parser, parseToken_t *parseToken);
	bool					(*Parse_WhiteSpace)(char chr);

	//containers
	cntArray_t				*(*Array_Alloc)(int elementSize, int initialNum);
	void					(*Array_Free)(cntArray_t *ar);
	void					(*Array_SetGrowth)(cntArray_t *ar, bool exponential);
	void					*(*Array_GetElement)(cntArray_t *ar, int index);
	void					*(*Array_GetElementGrow)(cntArray_t *ar, int index);
	void					(*Array_Append)(cntArray_t *ar, void *element);
	void					(*Array_RemoveLast)(cntArray_t *ar);
	void					(*Array_Insert)(cntArray_t *ar, void *element, int index);
	void					(*Array_Remove)(cntArray_t *ar, int index);
	int						(*Array_GetCount)(cntArray_t *ar);
	void					(*Array_Tighten)(cntArray_t *ar);
	unsigned int			(*Hash_GetKey)(unsigned char *data, int dataLen);
	cntHash_t				*(*Hash_Alloc)(int size);
	void					(*Hash_Free)(cntHash_t *hash);
	void					(*Hash_AddIndexedKey)(cntHash_t *hash, unsigned int key, int index);
	void					(*Hash_RemoveIndexedKey)(cntHash_t *hash, unsigned int key, int index);
	cntHashList_t			*(*Hash_GetKeyList)(cntHash_t *hash, unsigned int key);
	cntStream_t				*(*Stream_Alloc)(void *buffer, int size);
	void					(*Stream_Free)(cntStream_t *st);
	void					(*Stream_WriteBits)(cntStream_t *st, void *buf, int size);
	void					(*Stream_WriteBytes)(cntStream_t *st, void *buf, int size);
	bool					(*Stream_ReadBits)(cntStream_t *st, void *buf, int size);
	bool					(*Stream_ReadBytes)(cntStream_t *st, void *buf, int size);
	void					(*Stream_WriteBool)(cntStream_t *st, bool val);
	void					(*Stream_WriteInt)(cntStream_t *st, int val);
	void					(*Stream_WriteFloat)(cntStream_t *st, float val);
	void					(*Stream_WriteString)(cntStream_t *st, char *str);
	bool					(*Stream_ReadBool)(cntStream_t *st);
	int						(*Stream_ReadInt)(cntStream_t *st);
	float					(*Stream_ReadFloat)(cntStream_t *st);
	void					(*Stream_ReadString)(cntStream_t *st, char *str, int maxSize);
	void					*(*Stream_Buffer)(cntStream_t *st);
	int						(*Stream_Size)(cntStream_t *st);
	void					(*Stream_SetOffset)(cntStream_t *st, int offset);
	int						(*Stream_GetOffset)(cntStream_t *st);
	strHash_t				*(*SHash_Alloc)(int poolSize, int hashSize);
	void					(*SHash_Free)(strHash_t *s);
	int						(*SHash_Index)(strHash_t *s, char *str);
	int						(*SHash_IndexNoAdd)(strHash_t *s, char *str);
	char					*(*SHash_String)(strHash_t *s, int index);
	int						(*SHash_GetCount)(strHash_t *s);

	char					*(*TLoc_GetString)(const char *strid);
} sharedCLFunctions_t;

#endif //_CLSHARED_H
