/*
=============================================================================
Module Information
------------------
Name:			obj_ccon.cpp
Author:			Rich Whitehouse
Description:	server logic object: control console
=============================================================================
*/

#include "main.h"
#include "ai.h"

typedef enum
{
	DEMOGUYANIM_IDLE = 0,
	DEMOGUYANIM_PAIN1,
	DEMOGUYANIM_PAIN2,
	NUM_DEMOGUY_ANIMS
} demoGuyAnim_e;
static gameAnim_t g_demoGuyAnims[NUM_DEMOGUY_ANIMS] =
{
	{60, 100, 118.0f, true},			//DEMOGUYANIM_IDLE
	{172, 210, 16.0f, false},			//DEMOGUYANIM_PAIN1
	{243, 274, 16.0f, false}			//DEMOGUYANIM_PAIN2
};

//pick which anim to be in
void ObjDemoGuy_PickAnim(gameObject_t *obj, float timeMod)
{
	gameAnim_t *curAnim = obj->animTable+obj->curAnim;
	if (obj->net.frame == curAnim->endFrame && !obj->animRestart)
	{
		AI_StartAnim(obj, DEMOGUYANIM_IDLE, false);
	}
}

//think
void ObjDemoGuy_Think(gameObject_t *obj, float timeMod)
{
	Util_AnimateObject(obj, timeMod);
	if (obj->animhandler)
	{
		obj->animhandler(obj, timeMod);
	}
}

//pain
void ObjDemoGuy_Pain(gameObject_t *obj, gameObject_t *hurter, int dmg, const collObj_t *col)
{
	if (col)
	{
		float ang[3];
		Math_VecToAngles(col->endNormal, ang);
		ObjParticles_Create("melee/bleed", col->endPos, ang, -1);
		float z = col->endPos[2]-obj->net.pos[2];
		if (z > obj->net.maxs[2]*0.5f)
		{
			AI_StartAnim(obj, DEMOGUYANIM_PAIN1, true);
		}
		else
		{
			AI_StartAnim(obj, DEMOGUYANIM_PAIN2, true);
		}
	}
	if (hurter->plObj)
	{
		const invItemDef_t *item = ObjPlayer_GetEquippedItem(hurter);
		if (item)
		{
			if (item->itemBehavior == INVITEM_BEHAVIOR_VIEWHEALTH)
			{
				ObjPlayer_HealthDisplay(hurter, obj);
			}
		}
	}
}

//death
void ObjDemoGuy_Death(gameObject_t *obj, gameObject_t *killer, int dmg)
{
	if (obj->target && obj->target->inuse && obj->target->activate && !(obj->target->net.renderEffects & FXFL_HIDDEN))
	{
		obj->target->activate(obj->target, obj);
	}
	if (obj->chain)
	{
		obj->chain->think = ObjGeneral_RemoveThink;
		obj->chain->thinkTime = g_glb.gameTime;
		obj->chain = NULL;
	}
	float c[3];
	obj->hurtable = false;
	float ang[3] = {0.0f, 0.0f, 0.0f};
	Util_GameObjectCenter(obj, c);
	ObjParticles_Create("other/conblow", c, ang, -1);
	ObjSound_Create(c, "assets/sound/cb/boom.wav", 1.0f, -1);
	ObjSound_Create(c, "assets/sound/cb/boom.wav", 1.0f, -1);
	obj->net.renderEffects2 |= FXFL2_DEATH2;
	obj->think = ObjGeneral_RemoveThink;
	obj->thinkTime = g_glb.gameTime + 5000;
	obj->net.solid = 0;
	LServ_UpdateRClip(obj);

	if (killer && killer->inuse && killer->hurtable)
	{ //radius damage in the name of the killer
		Util_RadiusDamage(killer, 200, 1024.0f, c, false);
	}
}

//spawn
void ObjDemoGuy_Spawn(gameObject_t *obj, BYTE *b, const objArgs_t *args, int numArgs)
{
	g_sharedFn->Common_ServerString("&assets/models/chris.rdm");
	g_sharedFn->Common_ServerString("@assets/models/chris.rda");

	AI_GenericSpawn(obj, b, args, numArgs);

	obj->canBePushed = false;
	obj->aiObj->maxHealth = obj->health;
	obj->aiObj->nonActive = true;

	obj->net.aiDescIndex = AIDESC_E30BLUE;

	obj->think = ObjDemoGuy_Think;
	obj->pain = ObjDemoGuy_Pain;
	obj->death = ObjDemoGuy_Death;

	/*
	objArgs_t lightArgs[5];
	lightArgs[0].key = "color";
	lightArgs[0].val = "(1.5 1.5 1.5)";
	lightArgs[1].key = "radius";
	lightArgs[1].val = "2048";
	lightArgs[2].key = "shadows";
	lightArgs[2].val = "0";
	lightArgs[3].key = "spotcone";
	lightArgs[3].val = "60";
	lightArgs[4].key = "spotlight";
	lightArgs[4].val = "1";
	float lightAng[3] = {30.0f, obj->net.ang[YAW]+90.0f, 0.0f};
	float lightPos[3];
	float ang[3] = {obj->net.ang[0], obj->net.ang[1]-90.0f, 0.0f};
	float fwd[3], up[3];
	Math_AngleVectors(ang, fwd, 0, up);
	Math_VecMA(obj->net.pos, 700.0f, fwd, lightPos);
	Math_VecMA(lightPos, 500.0f, up, lightPos);
	obj->chain = LServ_ObjectFromName("obj_dynamiclight", lightPos, lightAng, lightArgs, 5);
	if (obj->chain)
	{
		obj->chain->spawnArgs = NULL;
		obj->chain->numSpawnArgs = 0;
	}
	*/
	obj->animhandler = ObjDemoGuy_PickAnim;
	obj->animTable = g_demoGuyAnims;
	obj->numAnims = NUM_DEMOGUY_ANIMS;
	obj->curAnim = DEMOGUYANIM_IDLE;
}
